@@ -5,6 +5,12 @@ from stackexchange.core import *
 # Site constants
 from stackexchange.sites import *
 
+def or_none(o, k):
+	try:
+		return getattr(o, k)
+	except:
+		return None
+
 ##### Statistics    ###
 class Statistics(JSONModel):
 	"""Stores statistics for a StackExchange site."""
@@ -27,11 +33,13 @@ class Answer(JSONModel):
 			comment = False
 		else:
 			comment = ('comment' in json._params_ and json._params_['comment'])
-		self.comments = site.build_from_snippet(json.comments, Comment) if comment else StackExchangeLazySequence(Comment, None, site, json.answer_comments_url, self._up('comments'))
+
+		answer_comments_url = 'answers/%d/comments' % self.id
+		self.comments = site.build_from_snippet(json.comments, Comment) if comment else StackExchangeLazySequence(Comment, None, site, answer_comments_url, self._up('comments'), filter = '!-*7AsUyrEan0')
 
 		self._question, self._owner = None, None
 		if hasattr(json, 'owner'):
-			self.owner_id = json.owner['user_id']
+			self.owner_id = json.owner.get('user_id')
 			self.owner_info = tuple(json.owner.values())
 
 		self.creation_date = datetime.datetime.fromtimestamp(json.creation_date)
@@ -41,8 +49,11 @@ class Answer(JSONModel):
 		if hasattr(json, 'last_activity_date'):
 			self.last_activity_date = datetime.date.fromtimestamp(json.last_activity_date)
 
-		self.revisions = StackExchangeLazySequence(PostRevision, None, site, 'revisions/%s' % self.id, self._up('revisions'), 'revisions')
-		self.votes = (self.up_vote_count, self.down_vote_count)
+		self.revisions = StackExchangeLazySequence(PostRevision, None, site, 'posts/%s/revisions' % self.id, self._up('revisions'), 'revisions')
+
+		if hasattr(self, 'up_vote_count') and hasattr(self, 'down_vote_count'):
+			self.votes = (self.up_vote_count, self.down_vote_count)
+
 		self.url = 'http://' + self.site.root_domain + '/questions/' + str(self.question_id) + '/' + str(self.id) + '#' + str(self.id)
 
 	def _get_user(self, id):
@@ -68,7 +79,7 @@ class Answer(JSONModel):
 		return site.answer(self.id)
 
 	def __unicode__(self):
-		return u'Answer %d [%s]' % (self.id, self.title)
+		return u'Answer %d' % self.id
 
 	def __str__(self):
 		return str(unicode(self))
@@ -83,21 +94,23 @@ class Question(JSONModel):
 	def _extend(self, json, site):
 		self.id = json.question_id
 
-		self.timeline = StackExchangeLazySequence(TimelineEvent, None, site, json.question_timeline_url, self._up('timeline'))
-		self.revisions = StackExchangeLazySequence(PostRevision, None, site, 'revisions/%s' % self.id, self._up('revisions'), 'revisions')
+		timeline_url = 'questions/%d/timeline' % self.id
+		self.timeline = StackExchangeLazySequence(TimelineEvent, None, site, timeline_url, self._up('timeline'))
+		self.revisions = StackExchangeLazySequence(PostRevision, None, site, 'posts/%s/revisions' % self.id, self._up('revisions'), 'revisions')
 
 		self.creation_date = datetime.datetime.fromtimestamp(json.creation_date)
-		self.comments_url = json.question_comments_url
-		self.comments = StackExchangeLazySequence(Comment, None, site, self.comments_url, self._up('comments'))
 
-		self.answers_url = json.question_answers_url
+		comments_url = 'questions/%d/comments' % self.id
+		self.comments = StackExchangeLazySequence(Comment, None, site, comments_url, self._up('comments'), filter = '!-*7AsUyrEan0')
+
+		self.answers_url = 'questions/%d/answers' % self.id
 
 		if hasattr(json, 'answers'):
 			self.answers = [Answer(x, site) for x in json.answers]
 		else:
 			self.answers = []
 
-		if hasattr(json, 'owner'):
+		if hasattr(json, 'owner') and 'user_id' in json.owner:
 			self.owner_id = json.owner['user_id']
 
 			owner_dict = dict(json.owner)
@@ -137,7 +150,7 @@ class Comment(JSONModel):
 				'user_type': Enumeration.from_string(json.owner['user_type'], UserType),
 				'display_name': json.owner['display_name'],
 				'reputation': json.owner['reputation'],
-				'email_hash': json.owner['email_hash']})
+				'profile_image': json.owner['profile_image']})
 		else:
 			self.owner = None
 
@@ -148,7 +161,7 @@ class Comment(JSONModel):
 				'user_type': Enumeration.from_string(json.reply_to['user_type'], UserType),
 				'display_name': json.reply_to['display_name'],
 				'reputation': json.reply_to['reputation'],
-				'email_hash': json.reply_to['email_hash']})
+				'profile_image': json.reply_to['profile_image']})
 
 		self.post_type = PostType.from_string(json.post_type)
 
@@ -186,7 +199,7 @@ class PostRevision(JSONModel):
 			'user_type': Enumeration.from_string(part['user_type'], UserType),
 			'display_name': part['display_name'],
 			'reputation': part['reputation'],
-			'email_hash': part['email_hash']
+			'profile_image': part['profile_image']
 		})
 
 	def _get_post(self):
@@ -210,29 +223,30 @@ class TagSynonym(JSONModel):
 
 	def _extend(self, json, site):
 		self.creation_date = datetime.datetime.fromtimestamp(json.creation_date)
-		self.last_applied_date = datetime.date.fromtimestamp(json.last_applied_date)
+		if hasattr(json, 'last_applied_date'):
+			self.last_applied_date = datetime.date.fromtimestamp(json.last_applied_date)
 
 	def __repr__(self):
 		return "<TagSynonym '%s'->'%s'>" % (self.from_tag, self.to_tag)
 
 class TagWiki(JSONModel):
-	transfer = ('tag_name')
+	transfer = ('tag_name', 'body', 'excerpt')
 
 	def _extend(self, json, site):
-		self.body = json.wiki_body
-		self.excerpt = json.wiki_excerpt
 		self.body_last_edit_date = datetime.date.fromtimestamp(json.body_last_edit_date)
 		self.excerpt_last_edit_date = datetime.date.fromtimestamp(json.excerpt_last_edit_date)
 
-		body_editor = dict(json.last_body_editor)
-		body_editor['id'] = body_editor['user_id']
-		del body_editor['user_id']
-		self.last_body_editor = User.partial(lambda s: s.site.user(self.id), site, body_editor)
+		if hasattr(json, 'last_body_editor'):
+			body_editor = dict(json.last_body_editor)
+			body_editor['id'] = body_editor['user_id']
+			del body_editor['user_id']
+			self.last_body_editor = User.partial(lambda s: s.site.user(self.id), site, body_editor)
 
-		excerpt_editor = dict(json.last_excerpt_editor)
-		excerpt_editor['id'] = excerpt_editor['user_id']
-		del excerpt_editor['user_id']
-		self.last_excerpt_editor = User.partial(lambda s: s.site.user(self.id), site, excerpt_editor)
+		if hasattr(json, 'last_excerpt_editor'):
+			excerpt_editor = dict(json.last_excerpt_editor)
+			excerpt_editor['id'] = excerpt_editor['user_id']
+			del excerpt_editor['user_id']
+			self.last_excerpt_editor = User.partial(lambda s: s.site.user(self.id), site, excerpt_editor)
 
 class Period(Enumeration):
 	AllTime, Month = 'all-time', 'month'
@@ -275,7 +289,9 @@ class Badge(JSONModel):
 	transfer = ('name', 'description', 'award_count', 'tag_based')
 	def _extend(self, json, site):
 		self.id = json.badge_id
-		self.recipients = StackExchangeLazySequence(User, None, site, json.badges_recipients_url, self._up('recipients'))
+
+		badges_recipients_url = 'badges/%d/recipients' % self.id
+		self.recipients = StackExchangeLazySequence(User, None, site, badges_recipients_url, self._up('recipients'))
 
 	def __str__(self):
 		return self.name
@@ -288,7 +304,8 @@ class RepChange(JSONModel):
 	transfer = ('user_id', 'post_id', 'post_type', 'title', 'positive_rep', 'negative_rep')
 	def _extend(self, json, site):
 		self.on_date = datetime.date.fromtimestamp(json.on_date)
-		self.score = self.positive_rep - self.negative_rep
+		if hasattr(json, 'positive_rep') and hasattr(json, 'negative_rep'):
+			self.score = json.positive_rep - json.negative_rep
 
 ## Timeline ##
 class TimelineEventType(Enumeration):
@@ -374,36 +391,59 @@ class TopTag(JSONModel):
 class User(JSONModel):
 	"""Describes a user on a StackExchange site."""
 
-	transfer = ('display_name', 'email_hash', 'age', 'website_url', 'location', 'about_me',
-		'view_count', 'up_vote_count', 'down_vote_count', 'association_id')
+	transfer = ('display_name', 'profile_image', 'age', 'website_url', 'location', 'about_me',
+		'view_count', 'up_vote_count', 'down_vote_count', 'account_id', 'profile_image')
 	def _extend(self, json, site):
 		self.id = json.user_id
-		self.type = Enumeration.from_string(json.user_type, UserType)
 		self.creation_date = datetime.datetime.fromtimestamp(json.creation_date)
 		self.last_access_date = datetime.date.fromtimestamp(json.last_access_date)
 		self.reputation = FormattedReputation(json.reputation)
 
-		self.questions = StackExchangeLazySequence(Question, json.question_count, site, json.user_questions_url, self._up('questions'))
+		# for compatibility reasons; this field name changed in v2.x
+		self.association_id = json.account_id
+
+		user_questions_url = 'users/%d/questions' % self.id
+		question_count = or_none(json, 'question_count')
+		self.questions = StackExchangeLazySequence(Question, question_count, site, user_questions_url, self._up('questions'))
+
+		user_favorites_url = 'users/%d/favorites' % self.id
+		self.favorites = StackExchangeLazySequence(Question, None, site, user_favorites_url, self._up('favorites'), 'questions')
+
 		self.no_answers_questions = StackExchangeLazySequence(Question, None, site, 'users/%d/questions/no-answers' % self.id, self._up('no_answers_questions'), 'questions')
 		self.unanswered_questions = StackExchangeLazySequence(Question, None, site, 'users/%d/questions/unanswered' % self.id, self._up('unanswered_questions'), 'questions')
 		self.unaccepted_questions = StackExchangeLazySequence(Question, None, site, 'users/%d/questions/unaccepted' % self.id, self._up('unaccepted_questions'), 'questions')
-		self.favorites = StackExchangeLazySequence(Question, None, site, json.user_favorites_url, self._up('favorites'), 'questions')
 
-		self.answers = StackExchangeLazySequence(Answer, json.answer_count, site, json.user_answers_url, self._up('answers'))
+		user_answers_url = 'users/%d/answers' % self.id
+		answer_count = or_none(json, 'answer_count')
+		self.answers = StackExchangeLazySequence(Answer, answer_count, site, user_answers_url, self._up('answers'))
+
 		# Grr, American spellings. Using them for consistency with official API.
-		self.tags = StackExchangeLazySequence(Tag, None, site, json.user_tags_url, self._up('tags'))
-		self.badges = StackExchangeLazySequence(Badge, None, site, json.user_badges_url, self._up('badges'))
-		self.timeline = StackExchangeLazySequence(TimelineEvent, None, site, json.user_timeline_url, self._up('timeline'), 'user_timelines')
-		self.reputation_detail = StackExchangeLazySequence(RepChange, None, site, json.user_reputation_url, self._up('reputation_detail'))
+		user_tags_url = 'users/%d/tags' % self.id
+		self.tags = StackExchangeLazySequence(Tag, None, site, user_tags_url, self._up('tags'))
 
-		self.mentioned = StackExchangeLazySequence(Comment, None, site, json.user_mentioned_url, self._up('mentioned'), 'comments')
-		self.comments = StackExchangeLazySequence(Comment, None, site, json.user_comments_url, self._up('comments'))
-		self.mentioned = StackExchangeLazySequence(Comment, None, site, 'users/%d/mentioned' % self.id, self._up('mentioned'))
+		user_badges_url = 'users/%d/badges' % self.id
+		self.badges = StackExchangeLazySequence(Badge, None, site, user_badges_url, self._up('badges'))
+
+		user_timeline_url = 'users/%d/timeline' % self.id
+		self.timeline = StackExchangeLazySequence(TimelineEvent, None, site, user_timeline_url, self._up('timeline'), 'user_timelines')
+
+		user_reputation_url = 'users/%d/reputation' % self.id
+		self.reputation_detail = StackExchangeLazySequence(RepChange, None, site, user_reputation_url, self._up('reputation_detail'))
+
+		user_mentioned_url = 'users/%d/mentioned' % self.id
+		self.mentioned = StackExchangeLazySequence(Comment, None, site, user_mentioned_url, self._up('mentioned'), 'comments')
+
+		user_comments_url = 'users/%d/comments' % self.id
+		self.comments = StackExchangeLazySequence(Comment, None, site, user_comments_url, self._up('comments'))
 
 		self.top_answer_tags = StackExchangeLazySequence(TopTag, None, site, 'users/%d/top-answer-tags' % self.id, self._up('top_answer_tags'), 'top_tags')
 		self.top_question_tags = StackExchangeLazySequence(TopTag, None, site, 'users/%d/top-question-tags' % self.id, self._up('top_question_tags'), 'top_tags')
 
-		self.vote_counts = (self.up_vote_count, self.down_vote_count)
+		if hasattr(self, 'up_vote_count') and hasattr(self, 'down_vote_count'):
+			self.vote_counts = (self.up_vote_count, self.down_vote_count)
+
+		self.type = Enumeration.from_string(json.user_type, UserType) if hasattr(json, 'user_type') else None
+
 
 		gold = json.badge_counts['gold'] if 'gold' in json.badge_counts else 0
 		silver = json.badge_counts['silver'] if 'silver' in json.badge_counts else 0
@@ -420,8 +460,8 @@ class User(JSONModel):
 
 		self.url = 'http://' + self.site.root_domain + '/users/' + str(self.id)
 
-	def has_privelege(self, privelege):
-		return self.reputation >= privelege.reputation
+	def has_privilege(self, privilege):
+		return self.reputation >= privilege.reputation
 
 	def _get_real_tag(self, tag):
 		return tag.name if isinstance(tag, Tag) else tag
@@ -443,7 +483,7 @@ class User(JSONModel):
 	def __repr__(self):
 		return "<User '%s' (%d) @ %x>" % (self.display_name, self.id, id(self))
 
-class Privelege(JSONModel):
+class Privilege(JSONModel):
 	transfer = ('short_description', 'description', 'reputation')
 
 
@@ -459,6 +499,30 @@ class QuestionsQuery(object):
 		if self.site.include_comments:
 			kw['comments'] = 'true'
 
+		# for API v2.x, the comments, body and answers parameters no longer
+		# exist; instead, we have to use filters. for now, take the easy way
+		# out and just rewrite them in terms of the new filters.
+		if 'filter' not in kw:
+			filter_name = '_'
+
+			if kw.get('body'):
+				filter_name += 'b'
+				del kw['body']
+			if kw.get('comments'):
+				filter_name += 'c'
+				del kw['comments']
+			if kw.get('answers'):
+				filter_name += 'a'
+				del kw['answers']
+
+			if filter_name == '_ca':
+				# every other compatibility filter name works in the above
+				# order except this one...
+				kw['filter'] = '_ac'
+			elif filter_name != '_':
+				kw['filter'] = filter_name
+
+
 	def __call__(self, ids=None, user_id=None, **kw):
 		self.check(kw)
 
@@ -517,18 +581,25 @@ class Site(object):
 	"""Stores information and provides methods to access data on a StackExchange site. This class is the 'root' of the API - all data is accessed
 through here."""
 
-	def __init__(self, domain, app_key=None, cache=1800):
+	def __init__(self, domain, app_key=None, cache=1800, impose_throttling=False):
 		self.domain = domain
 		self.app_key = app_key
-		self.api_version = '1.1'
+		self.api_version = '2.2'
 
-		self.impose_throttling = False
+		self.impose_throttling = impose_throttling
 		self.throttle_stop = True
 		self.cache_options = {'cache': False} if cache == 0 else {'cache': True, 'cache_age': cache}
 
 		self.include_body = False
 		self.include_comments = False
-		self.root_domain = '.'.join(self.domain.split('.')[1:])
+
+		# In API v2.x, we generally don't get given api. at the start of these things, nor are they
+		# strictly domains in many cases. We continue to accept api.* names for compatibility.
+		domain_components = self.domain.split('.')
+		if domain_components[0] == 'api':
+			self.root_domain = '.'.join(domain_components[1:])
+		else:
+			self.root_domain = domain
 
 	URL_Roots = {
 		User: 'users/%s',
@@ -542,7 +613,7 @@ through here."""
 		try:
 			if isinstance(ob, datetime.datetime):
 				return str(time.mktime(ob.timetuple()))
-			elif isinstance(ob, str):
+			elif isinstance(ob, basestring):
 				return ob
 			else:
 				i = iter(ob)
@@ -551,15 +622,19 @@ through here."""
 			return str(ob).lower()
 
 	def _request(self, to, params):
-		url = 'http://' + self.domain + '/' + self.api_version + '/' + to
+		url = 'http://api.stackexchange.com/' + self.api_version + '/' + to
+		params['site'] = params.get('site', self.root_domain)
 
 		new_params = {}
 		for k, v in params.iteritems():
-			if k in ('fromdate', 'todate'):
+			if v is None:
+				pass
+			elif k in ('fromdate', 'todate'):
 				# bit of a HACKish workaround for a reported issue; force to an integer
 				new_params[k] = str(int(v))
 			else:
 				new_params[k] = self._kw_to_str(v)
+
 		if self.app_key != None:
 			new_params['key'] = self.app_key
 
@@ -569,9 +644,10 @@ through here."""
 
 		json, info = request_mgr.json_request(url, new_params)
 
-		self.rate_limit = (int(info.getheader('X-RateLimit-Current')), int(info.getheader('X-RateLimit-Max')))
-		self.requests_used = self.rate_limit[1] - self.rate_limit[0]
-		self.requests_left = self.rate_limit[0]
+		if 'quota_remaining' in json and 'quota_max' in json:
+			self.rate_limit = (json['quota_remaining'], json['quota_max'])
+			self.requests_used = self.rate_limit[1] - self.rate_limit[0]
+			self.requests_left = self.rate_limit[0]
 
 		return json
 
@@ -607,12 +683,12 @@ through here."""
 		if allowed_types is not None and not hasattr(allowed_types, '__iter__'):
 			allowed_types = (allowed_types, )
 
-		if hasattr(lst, '__iter__'):
+		if isinstance(lst, basestring) or type(lst).__name__ == 'bytes':
+			return lst
+		elif hasattr(lst, '__iter__'):
 			return ';'.join([self.vectorise(x, or_of_type) for x in lst])
 		elif allowed_types is not None and any([isinstance(lst, typ) for typ in allowed_types]) and hasattr(lst, 'id'):
 			return str(lst.id)
-		elif isinstance(lst, str):
-			return lst
 		else:
 			return str(lst).lower()
 
@@ -698,22 +774,25 @@ unlike on the actual site, you will receive an error rather than a redirect to t
 		else:
 			return self._get(Badge, ids, 'users', kw)
 
-	def badge(self, nid, name=None, **kw):
+	def badge(self, nid = None, name=None, **kw):
 		"""Returns an object representing the badge with the ID 'nid', or with the name passed in as name=."""
-		if name is None:
-			b, = self.badges((nid,), kw)
+		if nid is not None and name is None:
+			b, = self.build('badges/%d' % nid, Badge, 'badges', kw)
 			return b
-		else:
+		elif nid is None and name is not None:
 			# We seem to need to get all badges and find it by name. Sigh.
-			all_badges = self.badges()
+			kw['inname'] = name
+			all_badges = self.build('badges', Badge, 'badges', kw)
 			for badge in all_badges:
 				if badge.name == name:
 					return badge
 			return None
+		else:
+			raise KeyError('Supply exactly one of the following: a badge ID, a badge name')
 
-	def privileges(self):
+	def privileges(self, **kw):
 		"""Returns all the privileges a user can have on the site."""
-		return self.build('privileges', Privelege, 'privileges', kw)
+		return self.build('privileges', Privilege, 'privileges', kw)
 
 	def all_nontag_badges(self, **kw):
 		"""Returns the set of all badges which are not tag-based."""
@@ -756,12 +835,16 @@ unlike on the actual site, you will receive an error rather than a redirect to t
 		return self.build('tags', Tag, 'tags', kw)
 
 	def tag(self, tag, **kw):
-		kw['filter'] = tag
-		return self.build('tags', Tag, 'tags', kw)[0]
+		return self.build('tags/%s/info' % tag, Tag, 'tags', kw)[0]
 
 	def tag_synonyms(self, **kw):
 		return self.build('tags/synonyms', TagSynonym, 'tag_synonyms', kw)
 
+	def error(self, id, **kw):
+		# for some reason, the SE API couldn't possible just ignore site=
+		kw['site'] = None
+		return self._request('errors/%d' % id, kw)
+
 	def __add__(self, other):
 		if isinstance(other, Site):
 			return CompositeSite(self, other)
