@@ -69,8 +69,8 @@ ITSecurity = __SEAPI('api.security.stackexchange.com')
 ITSecurityMeta = __SEAPI('api.meta.security.stackexchange.com')
 Writers = __SEAPI('api.writers.stackexchange.com')
 WritersMeta = __SEAPI('api.meta.writers.stackexchange.com')
-AudioRecordingandProduction = __SEAPI('api.audio.stackexchange.com')
-AudioRecordingandProductionMeta = __SEAPI('api.meta.audio.stackexchange.com')
+AudioVideoProduction = __SEAPI('api.avp.stackexchange.com')
+AudioVideoProductionMeta = __SEAPI('api.meta.avp.stackexchange.com')
 GraphicDesign = __SEAPI('api.graphicdesign.stackexchange.com')
 GraphicDesignMeta = __SEAPI('api.meta.graphicdesign.stackexchange.com')
 DatabaseAdministrators = __SEAPI('api.dba.stackexchange.com')
@@ -117,3 +117,15 @@ Travel = __SEAPI('api.travel.stackexchange.com')
 TravelMeta = __SEAPI('api.meta.travel.stackexchange.com')
 PersonalProductivity = __SEAPI('api.productivity.stackexchange.com')
 PersonalProductivityMeta = __SEAPI('api.meta.productivity.stackexchange.com')
+Cryptography = __SEAPI('api.crypto.stackexchange.com')
+CryptographyMeta = __SEAPI('api.meta.crypto.stackexchange.com')
+Literature = __SEAPI('api.literature.stackexchange.com')
+LiteratureMeta = __SEAPI('api.meta.literature.stackexchange.com')
+SignalProcessing = __SEAPI('api.dsp.stackexchange.com')
+SignalProcessingMeta = __SEAPI('api.meta.dsp.stackexchange.com')
+FrenchLanguageandUsage = __SEAPI('api.french.stackexchange.com')
+FrenchLanguageandUsageMeta = __SEAPI('api.meta.french.stackexchange.com')
+Christianity = __SEAPI('api.christianity.stackexchange.com')
+ChristianityMeta = __SEAPI('api.meta.christianity.stackexchange.com')
+Bitcoin = __SEAPI('api.bitcoin.stackexchange.com')
+BitcoinMeta = __SEAPI('api.meta.bitcoin.stackexchange.com')
