@@ -1,6 +1,7 @@
 # stackweb.py - Core classes for web-request stuff
 
-import urllib2, httplib, datetime, operator, StringIO, gzip, time, urllib
+from stackexchange.core import StackExchangeError
+import urllib2, httplib, datetime, operator, io, gzip, time, urllib, urlparse
 import datetime
 try:
 	import json
@@ -34,15 +35,24 @@ class WebRequestManager(object):
 		self.throttle_stop = throttle_stop
 		# Whether to use request caching.
 		self.do_cache = cache
-		# The time, in seconds, to cache a response
+		# The time, in seconds, for which to cache a response
 		self.cache_age = cache_age
+		# The time at which we should resume making requests after receiving a 'backoff' for each method
+		self.backoff_expires = {}
 	
-	window = None
+	# When we last made a request
+	window = datetime.datetime.now()
+	# Number of requests since last throttle window
 	num_requests = 0
 
 	def debug_print(self, *p):
 		if WebRequestManager.debug:
 			print ' '.join([x if isinstance(x, str) else repr(x) for x in p])
+	
+	def canon_method_name(self, url):
+		# Take the URL relative to the domain, without initial / or parameters
+		parsed = urlparse.urlparse(url)
+		return '/'.join(parsed.path.split('/')[1:])
 
 	def request(self, url, params):
 		now = datetime.datetime.now()
@@ -56,9 +66,10 @@ class WebRequestManager(object):
 			if not done:
 				url += '?'
 				done = True
-			else: url += '&'
+			else:
+				url += '&'
 
-			url += '%s=%s' % (k, urllib.quote(v.encode('utf-8')))
+			url += '%s=%s' % (k, urllib.quote(str(v).encode('utf-8')))
 		
 		# Now we have the `proper` URL, we can check the cache
 		if self.do_cache and url in self.cache:
@@ -70,17 +81,27 @@ class WebRequestManager(object):
 				return data
 
 		# Before we do the actual request, are we going to be throttled?
+		def halt(wait_time):
+			if self.throttle_stop:
+				raise TooManyRequestsError()
+			else:
+				# Wait the required time, plus a bit of extra padding time.
+				time.sleep(wait_time + 0.1)
+
 		if self.impose_throttling:
-			if (window - now).seconds >= 5:
-				window = now
-				num_requests = 0
-			num_requests += 1
-			if num_requests > 30:
-				if self.throttle_stop:
-					raise TooManyRequestsError()
-				else:
-					# Wait the required time, plus a bit of extra padding time.
-					time.sleep(5 - (window - now).seconds + 0.1)
+			# We need to check if we've been told to back off
+			method = self.canon_method_name(url)
+			backoff_time = self.backoff_expires.get(method, None)
+			if backoff_time is not None and backoff_time >= now:
+				self.debug_print('backoff: %s until %s' % (method, backoff_time))
+				halt((now - backoff_time).seconds)
+
+			if (now - WebRequestManager.window).seconds >= 5:
+				WebRequestManager.window = now
+				WebRequestManager.num_requests = 0
+			WebRequestManager.num_requests += 1
+			if WebRequestManager.num_requests > 30:
+				halt(5 - (WebRequestManager.window - now).seconds)
 
 		# We definitely do need to go out to the internet, so make the real request
 		self.debug_print('R>', url)
@@ -88,24 +109,40 @@ class WebRequestManager(object):
 		
 		request.add_header('Accept-encoding', 'gzip')
 		req_open = urllib2.build_opener()
-		conn = req_open.open(request)
 
-		req_data = conn.read()
+		try:
+			conn = req_open.open(request)
+			info = conn.info()
+			req_data = conn.read()
+			error_code = 200
+		except urllib2.HTTPError as e:
+			# we'll handle the error response later
+			error_code = e.code
+			# a hack (headers is an undocumented property), but there's no sensible way to get them
+			info = getattr(e, 'headers', {})
+			req_data = e.read()
 
 		# Handle compressed responses.
 		# (Stack Exchange's API sends its responses compressed but intermediary
 		# proxies may send them to us decompressed.)
-		if conn.info().getheader('Content-Encoding') == 'gzip':
-			data_stream = StringIO.StringIO(req_data)
+		if info.get('Content-Encoding') == 'gzip':
+			data_stream = io.BytesIO(req_data)
 			gzip_stream = gzip.GzipFile(fileobj=data_stream)
 
 			actual_data = gzip_stream.read()
 		else:
 			actual_data = req_data
 
-		info = conn.info()
-		conn.close()
+		# Check for errors
+		if error_code != 200:
+			try:
+				error_ob = json.loads(actual_data.decode('utf8'))
+			except:
+				raise StackExchangeError()
+			else:
+				raise StackExchangeError(error_ob.get('error_id', StackExchangeError.UNKNOWN), error_ob.get('error_name'), error_ob.get('error_message'))
 
+		conn.close()
 		req_object = WebRequest(actual_data, info)
 
 		# Let's store the response in the cache
@@ -117,5 +154,12 @@ class WebRequestManager(object):
 	
 	def json_request(self, to, params):
 		req = self.request(to, params)
-		return (json.loads(req.data), req.info)
+		parsed_result = json.loads(req.data.decode('utf8'))
+
+		# In API v2.x we now need to respect the 'backoff' warning
+		if 'backoff' in parsed_result:
+			method = self.canon_method_name(to)
+			self.backoff_expires[method] = datetime.datetime.now() + parsed_result[backoff]
+
+		return (parsed_result, req.info)
 
